/* 
   readline_prompt.c: prompts the user using readline
   Copyright (C) 2007, 2008 Vincent Fourmond

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdio.h>
#include <string.h>
#include <readline/readline.h>

#ifdef HAVE_READLINE_HISTORY_H
#  include <readline/history.h>
#endif

#include <cfortran/cfortran.h>

#include "commands.h"

const char * prompt_for(char * prompt, const char * context);

FCALLSCFUN2(STRING, prompt_for,PROMPTFOR,promptfor,STRING, STRING)

/* A static variable for holding the line. */
static char *line_read = (char *)NULL;

/* Read a string, and return a pointer to it.
   Returns NULL on EOF. */
char * rl_gets (const char * prompt)
{
  /* We save the previous line */
  char *last_read = line_read;
  
  /* Get a line from the user. */
  line_read = readline (prompt);
  
  /* If the line has any text in it, and is bigger than one single
     character, we save it on the history. We should first check
     if that isn't one of the last elements in history as well, to avoid 
     adding things that are already there...
  */
#ifdef HAVE_READLINE_HISTORY_H
  if (line_read && *line_read && (strlen(line_read) >= SOAS_HISTORY_SIZE) &&
      (! last_read || strcmp(line_read, last_read)) )
    add_history (line_read);
#endif

  /* If the buffer has already been allocated,
     return the memory to the free pool. */
  if(last_read)
    free(last_read);
  
  return (line_read);
}

/* Declaration of the completion function */
char ** soas_cmd_completion (const char *text, int start, int end);


/* 
   Sends a readline prompt. Returns the string given by
   the user. context, although not currently used, might
   get used one day to specify what kind of things are
   expected, so we could tune slightly automatic completion:
   
   * 'f' = a file
   * 'd' = a directory
   * 'c' = a command
   * 's' = a short command (such as for broWsing)
   * 'b' = a buffer name
   * 'e' = an expression (for fit_arb)
   * 'n' = a number
   
 */
const char * prompt_for(char * prompt, const char * context)
{

  switch(context[0]) 
    {
    case 'c':
      /* We use command completion */
      rl_attempted_completion_function = soas_cmd_completion;
      break;
    }

  char * r = rl_gets(prompt);
  /* We come back to default completion */
  rl_attempted_completion_function = NULL;

  /* If we are in the 'command' context and we get an end-of-file
     we return an exit command.
  */
  if(! r)
    {
      switch(context[0])
	{
	case 'c':
	  return "exit";
	case 's':
	  return "q";
	case 'b':
	  return "0";
	}
    }
  return r;
}

int prompt_for_int(char * , const char * );

FCALLSCFUN2(INT,prompt_for_int,PROMPTFORINT,promptforint,STRING,STRING)

/* The same function as prompt_for, except that we ask an integer */
int prompt_for_int(char * prompt, const char * context)
{
  return atoi(prompt_for(prompt, context));
}



float prompt_for_float(char * , const char * );

FCALLSCFUN2(FLOAT,prompt_for_float,PROMPTFORFLOAT,promptforfloat,STRING,STRING)

/* The same function as prompt_for, except that we ask a float */
float prompt_for_float(char * prompt, const char * context)
{
  return (float) atof(prompt_for(prompt, context));
}

/* Parts of the code underneath are adapted from the GNU Readline
   documentation.
*/


/* Generator function for command completion.  STATE lets us
   know whether to start from scratch; without any state
   (i.e. STATE == 0), then we start at the top of the list. */
char * soas_command_generator (const char * text, int state)
{
  static int list_index, len;
  char *name;
  
  /* If this is a new word to complete, initialize now.  This
     includes saving the length of TEXT for efficiency, and
     initializing the index variable to 0. */
  if (!state)
    {
      list_index = 0;
      len = strlen (text);
    }

  /* Return the next name which partially matches from the
     command list. */
  while (name = commands[list_index].cmd)
    {
      list_index++;
      
      if (strncmp (name, text, len) == 0)
	return (strdup(name));
    }
  
  /* If no names matched, then return NULL. */
  return NULL;
}


/* Attempt to complete on the contents of TEXT.  START and END
   bound the region of rl_line_buffer that contains the word to
   complete.  TEXT is the word to complete.  We can use the entire
   contents of rl_line_buffer in case we want to do some simple
   parsing.  Return the array of matches, or NULL if there aren't any. */ 
char ** soas_cmd_completion (const char *text, int start, int end)
{
  char **matches;

  matches = (char **)NULL;

  /* If this word is at the start of the line, then it is a command
     to complete.  Otherwise it is the name of a file in the current
     directory. */
  if (start == 0)
    matches = rl_completion_matches (text, soas_command_generator);
  
  return (matches);
}
