/* 
   soas_scalc.cc: dialog with scalc.cc
   Copyright (C) 2007, 2008 Vincent Fourmond

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include <scalc/scalc.hh>
#include <cfortran/cfortran.h>
#include <algorithm>

static SCalc::Session * session = 0;

static SCalc::Expression * expression = 0;

// The list of 'constants'
static std::set<int> constants;
// Their description
static std::map<int, std::string> constants_descriptions;

static int register_constant(const char * name, const char * desc, double val)
{
  int i = session->register_varname(name);
  constants.insert(i);
  constants_descriptions[i] = std::string(desc);
  session->set_var(i, val);
  return i;
}


// Initialization of the session, including the registration
// of various important parameters.
void scalcinit()
{
  if(session)
    return; 			// Nothing to do
  session = new SCalc::Session();

  // Now, we register common constants:
  register_constant("PI", "The constant PI", 3.1415926536);
  register_constant("T", "The absolute temperature", 0);
  register_constant("F", "The faraday constant",96485.3383 );
  register_constant("f", "F/RT", 0);
}

FCALLSCSUB0(scalcinit,SCALCINIT,scalcinit)

int scalcparse(const char * expr)
{
  if(!session)
    scalcinit();

  SCalc::ParserResult * res  = session->eval(expr);
  if(res->is_expression() && res->to_expression()->is_valid())
    {
      if(expression) 
	delete expression;
      expression = res->to_expression();
      return 1;
    }
  else 
    {
      std::cout << "There was a problem with your expression: " << std::endl
		<< res->pretty_print() << std::endl
		<< "Aborting" << std::endl;
      delete res;
      return 0;
    }
}

FCALLSCFUN1(LOGICAL,scalcparse,SCALCPARSE,scalcparse,STRING);

static std::set<int> parameters;


// Returns the number of parameters other than x in the expression.
int scalcparamnumber()
{
  if(expression) 
    {
      std::set<int> a = constants;
      std::set<int> used_vars;
      a.insert(session->register_varname("x"));

      // a = constants + x;

      used_vars = expression->used_variables();
      parameters.clear(); 	// remove previous ones.
      set_difference(used_vars.begin(), used_vars.end(),
		     a.begin(), a.end(), 
		     inserter(parameters,parameters.begin()));
      return parameters.size();
    }
  else
    return -1;			// Shouldn't happen, really !
}

FCALLSCFUN0(INT,scalcparamnumber,SCALCPARAMNUMBER,scalcparamnumber);

// The const_cast trick is necessary for cfortran. From what
// I remember, cfortran is responsible for handling the copying
// anyway.
char * scalcparametername(int n)
{
  n --; 			// 1-based indices...
  std::set<int>::iterator i = parameters.begin();
  int c = 0;
  while(i != parameters.end()) 
    {
      if(c == n)
	return const_cast<char *>(session->varname(*i));
      i++; c++;
    }
  return const_cast<char *>("problem...");
}

FCALLSCFUN1(STRING,scalcparametername,SCALCPARAMETERNAME,
	    scalcparametername, INT);



void scalc_compute(int n, void * x_p, void * y_p, void * p_p)
{
  float * x = (float *) x_p;
  float * y = (float *) y_p;
  float * params = (float *) p_p;
  int x_arg = session->register_varname("x");
  std::set<int>::iterator it = parameters.begin();
  int c = 0;
  double val[session->nb_vars_defined()];
  session->fill_default(val); // fill up with constants
  while(it != parameters.end()) 
    {
      val[*it] = params[c++];
      it++;
    }
  int i = 0;
  for(; i < n; i++)
    {
      val[x_arg] = x[i];
      y[i] = expression->evaluate(val);
    }
  return;
  //  delete val;
}

FCALLSCSUB4(scalc_compute,SCALCCOMPUTE,scalccompute,INT,FLOATV,
	    FLOATVV, FLOATV);

// Prints out the currently known 'constants'
void scalcprintvars()
{
  std::set<int>::iterator i = constants.begin();
  while(i != constants.end()) 
    {
      std::cout << session->varname(*i) << " : " 
		<< constants_descriptions[*i] << std::endl;
      
      i++;
    }
}

FCALLSCSUB0(scalcprintvars,SCALCPRINTVARS,scalcprintvars);

void scalcsettemperature(float t)
{
  double f = 96485./8.3144/t;
  session->set_var("T", t);
  session->set_var("f", f);
}

FCALLSCSUB1(scalcsettemperature, SCALCSETTEMPERATURE,
	    scalcsettemperature, FLOAT); 

